// P_2_1_2_03.pde
// 
// Generative Gestaltung, ISBN: 978-3-87439-759-9
// First Edition, Hermann Schmidt, Mainz, 2009
// Hartmut Bohnacker, Benedikt Gross, Julia Laub, Claudius Lazzeroni
// Copyright 2009 Hartmut Bohnacker, Benedikt Gross, Julia Laub, Claudius Lazzeroni
//
// http://www.generative-gestaltung.de
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * changing size of circles in a rad grid depending the mouseposition
 * 	 
 * MOUSE
 * position x/y        : module size and offset z
 * 
 * KEYS
 * s                   : save png
 * p                   : save pdf
 
 
 for (int gridY=0; gridY<width; gridY+=25) {
    for (int gridX=0; gridX<height; gridX+=25) {

      float diameter = dist(mouseX, mouseY, gridX, gridY);
      diameter = diameter/max_distance * 40;
      pushMatrix();
      translate(gridX, gridY, diameter*5);
      rotate(diameter/100*random(0,10));
      noStroke();
      fill(255);
      //ellipse(0, 0, diameter, diameter);
      fill(255);
      //rect(-diameter,-diameter, diameter*.25,diameter);
      //line(-diameter,-diameter, diameter,diameter);
      popMatrix(); 
    }
  }
 
 */

import SimpleOpenNI.*;

class HandCapture
{
  SimpleOpenNI  _context;
  boolean       _drawFlag=false;
  PVector       _handVec = new PVector();
  String        _lastGesture = "";
  PVector       _screenPos = new PVector();
  PApplet       _parent;
  boolean       _handsTrackFlag = false;

  HandCapture(PApplet parent)
  {
    _parent = parent;

    _context = new SimpleOpenNI(_parent);

    // mirror is by default enabled
    _context.setMirror(true);

    // enable depthMap 
    _context.enableDepth();

    // enable camera
    _context.enableRGB();

    _context.enableHands();  
    _context.enableGesture();

    _context.addGesture("RaiseHand");

    // align depth data to image data
    _context.alternativeViewPointDepthToImage();

  }

  boolean isHandTracked() { return _handsTrackFlag; } 
  
  PVector handPos() { return _screenPos; }
  PVector handPos3d() { return _handVec; }
  
  PImage depthImage() { return _context.depthImage(); }
  PImage rgbImage() { return _context.rgbImage(); }
  

  int width() { return _context.depthWidth(); }
  int height() { return _context.depthHeight(); }
  
  void update()
  {
    _context.update();
    
    _context.convertRealWorldToProjective(_handVec,_screenPos);
  }

  void onCreateHands(int handId, PVector pos, float time)
  {
    _handsTrackFlag = true;
    _handVec = pos;
  }

  void onUpdateHands(int handId, PVector pos, float time)
  {
    _handVec = pos;
  }

  void onDestroyHands(int handId, float time)
  {
    _handsTrackFlag = false;
    _context.addGesture(_lastGesture);
  }

  void onRecognizeGesture(String strGesture, PVector idPosition, PVector endPosition)
  {
    _lastGesture = strGesture;
    _context.removeGesture(strGesture); 
    _context.startTrackingHands(endPosition);
  }

  void onProgressGesture(String strGesture, PVector position, float progress)
  {
  }
}

